using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Text;

namespace GDIPac
{
	/// <summary>
	/// 
	/// </summary>
	public class Game
	{
		// This class is a Singleton
		#region Singleton Stuff
		private static Game _instance=null;
		public static Game GetInstance()
		{
			if(_instance == null)
				_instance = new Game();

			return _instance;
		}
		#endregion
		private Game()
		{
			LoadGameOverImage();
		}

		private PictureBox gameField;
		private Label lblScore;
		private Label lblLives;
		private Keyboard keys = Keyboard.GetInstance();
		private Playfield playfield;
		private Pac player;
		private Ghost[] ghosts = new Ghost[4];
		private Image gameOverImg;

		private const int powerPelletFullPowerDelay=40;
		private const int powerPelletLowPowerDelay=20;
		private int powerPelletTimer=0;

		private bool _isRunning=false;
		public bool IsRunning
		{
			get{ return _isRunning; }
		}

		public int Score
		{
			get{ return Int32.Parse(lblScore.Text); }
			set{ lblScore.Text = value.ToString(); }
		}

		public int Lives
		{
			get{ return Int32.Parse(lblLives.Text); }
			set{ lblLives.Text = value.ToString(); }
		}

		public void Start(PictureBox gameField, Label lblScore, Label lblLives)
		{
			this.gameField = gameField;
			this.lblScore = lblScore;
			this.lblLives = lblLives;
            
			playfield = new Playfield(gameField.Handle);
			InitSprites();

			Lives = 3;
			Score = 0;
			_isRunning = true;

			RedrawScreen();
		}

		public void Stop()
		{
			_isRunning = false;
		}

		private void InitSprites()
		{
			player = new Pac(gameField.Handle, playfield);
			for(int i=0; i<4; i++)
			{
				ghosts[i] = new Ghost(gameField.Handle, playfield, i,  7+i, 8);
			}
//			ghosts[0] = new Ghost(gameField.Handle, playfield, 0,  1, 10);
//			ghosts[1] = new Ghost(gameField.Handle, playfield, 1,  8, 12);
//			ghosts[2] = new Ghost(gameField.Handle, playfield, 2,  2,  1);
//			ghosts[3] = new Ghost(gameField.Handle, playfield, 3, 14,  5);
		}

		public void RedrawScreen()
		{
			if(IsRunning)
			{
				playfield.DrawFull();

				foreach(Ghost ghost in ghosts)
					ghost.Draw();
				player.Draw();
			}
		}

		  // Form1 calls this once every timer tick
		public void DoFrame()
		{

			if(IsRunning)
			{
				//DisplayKeyStatus();

				  // Update Sprites
				foreach(Ghost ghost in ghosts)
					ghost.DoFrame();
				player.DoFrame();

				  // If Level Cleared, Reset Level
				if(playfield.NumPellets <= 0)
				{
					playfield.Reset();
					playfield.DrawFull();
					InitSprites();
				}

				  // Handle Power Pellet Timer
				if(powerPelletTimer > 0)
				{
					powerPelletTimer--;

					if(powerPelletTimer == powerPelletLowPowerDelay)
						MakeGhostsRecovering();

					if(powerPelletTimer <= 0)
					{
						MakeGhostsAlive();
						player.SetAnimSlow();
					}
				}

				  // Handle Player/Ghost collisions
				foreach(Ghost ghost in ghosts)
				{
					if(Util.IsCollision(ghost, player))
					{
						if(ghost.State == Ghost.GhostState.Edible ||
						   ghost.State == Ghost.GhostState.Recovering)
						{
							ghost.State = Ghost.GhostState.Eaten;
							Score += 25;
						}
						else if(ghost.State == Ghost.GhostState.Alive)
						{
							Lives--;
							if(Lives <= 0)
							{
								 // Set it to zero in case more than one ghost has hit Pac this frame
								Lives = 0;
								Stop();
							}
							else
							{
								InitSprites();
								playfield.DrawFull();
							}
						}
					}
				}

				  // Erase Sprites
				foreach(Ghost ghost in ghosts)
					ghost.Erase();
				player.Erase();

				  // Draw Sprites
				foreach(Ghost ghost in ghosts)
					ghost.Draw();
				player.Draw();
			}
		}

		private void MakeGhostsEdible()
		{
			foreach(Ghost ghost in ghosts)
			{
				if(ghost.State == Ghost.GhostState.Alive || ghost.State == Ghost.GhostState.Recovering)
					ghost.State = Ghost.GhostState.Edible;
			}
		}

		private void MakeGhostsRecovering()
		{
			foreach(Ghost ghost in ghosts)
			{
				if(ghost.State == Ghost.GhostState.Edible)
					ghost.State = Ghost.GhostState.Recovering;
			}
		}

		private void MakeGhostsAlive()
		{
			foreach(Ghost ghost in ghosts)
			{
				if(ghost.State == Ghost.GhostState.Recovering)
					ghost.State = Ghost.GhostState.Alive;
			}
		}

		public void StartPowerPelletMode()
		{
			powerPelletTimer = powerPelletFullPowerDelay + powerPelletLowPowerDelay;
            MakeGhostsEdible();
		}

		private void LoadGameOverImage()
		{
			// Load Game Over Image
			Bitmap bmp;
			bmp = (Bitmap)Bitmap.FromFile("gameOver.bmp");
			bmp.MakeTransparent(Color.FromArgb(192, 192, 192));
			gameOverImg = bmp;
		}

		public void DisplayGameOver()
		{
			using(Graphics graphBack = Graphics.FromHwnd(gameField.Handle))
			{
				graphBack.DrawImageUnscaled(gameOverImg, 0, 0);
			}
		}

		// This for debugging purposes
		private void DisplayKeyStatus()
		{
			StringBuilder keyDisplay = new StringBuilder();
			
			if(keys.KeyUp)
				keyDisplay.Append("U");
			else
				keyDisplay.Append("x");
			
			if(keys.KeyDown)
				keyDisplay.Append("D");
			else
				keyDisplay.Append("x");
			
			if(keys.KeyLeft)
				keyDisplay.Append("L");
			else
				keyDisplay.Append("x");
			
			if(keys.KeyRight)
				keyDisplay.Append("R");
			else
				keyDisplay.Append("x");

			lblScore.Text = keyDisplay.ToString();
		}
	}
}
