using System;
using System.Collections;

namespace GDIPac
{
	/// <summary>
	/// Summary description for Ghost.
	/// </summary>
	public class Ghost : Sprite
	{
		private static ImageSet[] ghostImageSet;
		private static ImageSet ghostEatenImageSet;
		private static ImageSet ghostEdibleImageSet;
		private int ghostNum;

		public enum GhostState {Alive, Edible, Recovering, Eaten};

		private GhostState _state;
		public GhostState State
		{
			get { return _state; }
			set
			{
				_state = value;
				switch(_state)
				{
					case GhostState.Alive:
						SetImageSet(ghostImageSet[ghostNum]);
						SetAnimFast();
						break;

					case GhostState.Edible:
						SetImageSet(ghostEdibleImageSet);
						SetAnimSlow();
						break;

					case GhostState.Eaten:
						SetImageSet(ghostEatenImageSet);
						SetAnimSlow();
						break;

					case GhostState.Recovering:
						// Don't set imageset, DoExtraProcessing() will handle that
						SetAnimSlow();
						break;
				}
			}
		}

		public Ghost(IntPtr picBoxHandle, Playfield playfield, int ghostNum, int x, int y)
			: base(picBoxHandle, playfield, null, Direction.West, true, true)
		{
			this.ghostNum = ghostNum;

			if(ghostImageSet == null)
			{
				ghostImageSet = new ImageSet[4] {new ImageSet(), new ImageSet(),
												 new ImageSet(), new ImageSet()};
				for(int i=0; i<4; i++)
				{
					ghostImageSet[i].SetImage(Direction.North, "enemy" + (i+1) + "up1.bmp",    "enemy" + (i+1) + "up2.bmp");
					ghostImageSet[i].SetImage(Direction.South, "enemy" + (i+1) + "down1.bmp",  "enemy" + (i+1) + "down2.bmp");
					ghostImageSet[i].SetImage(Direction.East,  "enemy" + (i+1) + "right1.bmp", "enemy" + (i+1) + "right2.bmp");
					ghostImageSet[i].SetImage(Direction.West,  "enemy" + (i+1) + "left1.bmp",  "enemy" + (i+1) + "left2.bmp");
				}
			}

			if(ghostEatenImageSet == null)
			{
				ghostEatenImageSet = new ImageSet();

				ghostEatenImageSet.SetImage(Direction.North, "enemyEatenUp1.bmp",   "enemyEatenUp2.bmp");
				ghostEatenImageSet.SetImage(Direction.South, "enemyEatenUp1.bmp",   "enemyEatenUp2.bmp");
				ghostEatenImageSet.SetImage(Direction.East,  "enemyEatenLeft1.bmp", "enemyEatenLeft2.bmp");
				ghostEatenImageSet.SetImage(Direction.West,  "enemyEatenLeft1.bmp", "enemyEatenLeft2.bmp");
			}

			if(ghostEdibleImageSet == null)
			{
				ghostEdibleImageSet = new ImageSet();

				ghostEdibleImageSet.SetImage(Direction.North, "enemyEdibleUp1.bmp",    "enemyEdibleUp2.bmp");
				ghostEdibleImageSet.SetImage(Direction.South, "enemyEdibleDown1.bmp",  "enemyEdibleDown2.bmp");
				ghostEdibleImageSet.SetImage(Direction.East,  "enemyEdibleRight1.bmp", "enemyEdibleRight2.bmp");
				ghostEdibleImageSet.SetImage(Direction.West,  "enemyEdibleLeft1.bmp",  "enemyEdibleLeft2.bmp");
			}

			State = GhostState.Alive;

			X = x;
			Y = y;
		}

		private ArrayList GetAvailableDirections()
		{
			ArrayList directionList = new ArrayList();
			Util.TileActingDelgateB isWall = new Util.TileActingDelgateB(playfield.IsWall);
			Util.TileActingDelgateB isGhostDoor = new Util.TileActingDelgateB(playfield.IsGhostDoor);

			foreach( Direction direction in Enum.GetValues(typeof(Direction)) )
			{
				  // Don't try to go in through ghost door
				if(direction == Direction.South)
				{
					if(Util.ApplyAdjacentB(isGhostDoor, X, Y, direction))
						continue;
				}

				if(direction != Direction.None)
				{
					if(!Util.ApplyAdjacentB(isWall, X, Y, direction))
						directionList.Add(direction);
				}
			}

			return directionList;
		}

		private Direction RandomAvailableDirection()
		{
			ArrayList availableDirections = GetAvailableDirections();
			Direction reverseDir = Util.GetReverseDirection(CurrDirection);

			if(availableDirections.Contains(reverseDir))
				availableDirections.Remove(reverseDir);

			if(availableDirections.Count == 1)
				return (Direction)availableDirections[0];
			else
			{
				return (Direction)availableDirections[Util.random.Next(availableDirections.Count)];
			}
		}

		protected override Direction ChooseNextDirection()
		{
			Direction returnDirection;
			Util.TileActingDelgateB isGhostDoor = new Util.TileActingDelgateB(playfield.IsGhostDoor);

			if(State == GhostState.Eaten)
			{
				returnDirection = playfield.GetGhostReturnDir(X, Y);
				if(returnDirection == Direction.None)
				{
					State = GhostState.Alive;
					return Direction.East;
				}

				return returnDirection;
			}
			else
			{
				if( playfield.IsGhostDoor(X, Y) )
				{
					return Direction.North;
				}
				else
				{
					return RandomAvailableDirection();
				}
			}
		}

		protected override void DoExtraProcessing(bool isAnimAtZero)
		{
			if(State == Ghost.GhostState.Recovering)
			{
				if(isAnimAtZero)
				{
					SetImageSet(ghostImageSet[ghostNum]);
				}
				else
				{
					SetImageSet(ghostEdibleImageSet);
				}
			}
		}
	}
}
