using System;
using System.IO;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Text;

namespace GDIPac
{
	/// <summary>
	/// Summary description for Playfield.
	/// </summary>
	public class Playfield
	{
		 // NOTE: The order of these must not change
		enum Tile {empty=0, ghostDoor, wallSingle,
			wallN,   wallS,   wallE,   wallW,
			wallNS,  wallEW,
			wallNE,  wallNW,  wallSE,  wallSW,
			wallNSE, wallNSW, wallEWN, wallEWS,
			pellet,  powerPellet
		};

		private const int width=17;
		private const int height=16;

		private Direction[,] ghostReturnData = new Direction[width, height];
		private int[,] playfieldInitData = new int[width, height];
		private int[,] playfieldData;

		private int initNumPellets;
		private int numPellets;
		public int NumPellets
		{
			get{ return numPellets; }
			set{ numPellets = value; }
		}

		private Image[] tileImages = new Image[19];
		private Image[] directionImages = new Image[4];
		private IntPtr picBoxHandle;

		public Playfield(IntPtr picBoxHandle)
		{
			this.picBoxHandle = picBoxHandle;

			LoadMapData();
			LoadImages();
			Reset();

			//OutputMapData();
		}

		private void LoadMapData()
		{
			uint ghostReturnVal;

			initNumPellets = 0;
			using(StreamReader sr = new StreamReader("mapdata.dat"))
			{
				for(int y=0; y<height; y++)
				{
					for(int x=0; x<width; x++)
					{
						playfieldInitData[x,y] = sr.Read();
						if(playfieldInitData[x,y] == (int)Tile.pellet || playfieldInitData[x,y] == (int)Tile.powerPellet)
							initNumPellets++;

						ghostReturnVal = (uint)sr.Read();
						if(ghostReturnVal <= 4)
							ghostReturnData[x,y] = (Direction)ghostReturnVal;
						else
							ghostReturnData[x,y] = Direction.None;
					}
				}
			}
		}

		  // For debugging purposes only
		private void OutputMapData()
		{
			using(StreamWriter sw = new StreamWriter("mapdataout.txt"))
			{
				for(int y=0; y<height; y++)
				{
					for(int x=0; x<width; x++)
					{
						if(playfieldInitData[x,y] == 0)
							sw.Write("   ");
						else if(playfieldInitData[x,y] < 10)
							sw.Write(" " + playfieldInitData[x,y].ToString() + " ");
						else
							sw.Write(playfieldInitData[x,y].ToString() + " ");
					}
					sw.WriteLine();
				}
				sw.WriteLine();
				for(int y=0; y<height; y++)
				{
					for(int x=0; x<width; x++)
					{
						sw.Write(ghostReturnData[x,y].ToString() + " ");
					}
					sw.WriteLine();
				}
			}
		}

		private void LoadImages()
		{
			tileImages[(int)Tile.empty] = Bitmap.FromFile("empty.bmp");
			tileImages[(int)Tile.ghostDoor] = Bitmap.FromFile("ghostDoor.bmp");
			tileImages[(int)Tile.wallSingle] = Bitmap.FromFile("wallSingle.bmp");

			tileImages[(int)Tile.wallN] = Bitmap.FromFile("wallN.bmp");
			tileImages[(int)Tile.wallS] = Bitmap.FromFile("wallS.bmp");
			tileImages[(int)Tile.wallE] = Bitmap.FromFile("wallE.bmp");
			tileImages[(int)Tile.wallW] = Bitmap.FromFile("wallW.bmp");

			tileImages[(int)Tile.wallNS] = Bitmap.FromFile("wallNS.bmp");
			tileImages[(int)Tile.wallEW] = Bitmap.FromFile("wallEW.bmp");

			tileImages[(int)Tile.wallNE] = Bitmap.FromFile("wallNE.bmp");
			tileImages[(int)Tile.wallNW] = Bitmap.FromFile("wallNW.bmp");
			tileImages[(int)Tile.wallSE] = Bitmap.FromFile("wallSE.bmp");
			tileImages[(int)Tile.wallSW] = Bitmap.FromFile("wallSW.bmp");

			tileImages[(int)Tile.wallNSE] = Bitmap.FromFile("wallNSE.bmp");
			tileImages[(int)Tile.wallNSW] = Bitmap.FromFile("wallNSW.bmp");
			tileImages[(int)Tile.wallEWN] = Bitmap.FromFile("wallEWN.bmp");
			tileImages[(int)Tile.wallEWS] = Bitmap.FromFile("wallEWS.bmp");
			
			tileImages[(int)Tile.pellet]      = Bitmap.FromFile("pellet.bmp");
			tileImages[(int)Tile.powerPellet] = Bitmap.FromFile("powerPellet.bmp");

			directionImages[(int)Direction.North] = Bitmap.FromFile("n.bmp");
			directionImages[(int)Direction.South] = Bitmap.FromFile("s.bmp");
			directionImages[(int)Direction.East]  = Bitmap.FromFile("e.bmp");
			directionImages[(int)Direction.West]  = Bitmap.FromFile("w.bmp");
		}

		public void DrawFull()
		{
			for(int y=0; y<height; y++)
			{
				for(int x=0; x<width; x++)
				{
					DrawAt(x,y);
				}
			}
		}

		public void DrawAt(int x, int y)
		{
			using(Graphics graphBack = Graphics.FromHwnd(picBoxHandle))
			{
				// Uncomment these three lines to display Ghost Return Data
//				if(/*playfieldData[x,y] == (int)Tile.empty && */(int)ghostReturnData[x,y] < 4)
//					graphBack.DrawImageUnscaled(directionImages[(int)ghostReturnData[x,y]], x * 16, y*16);
//				else
					graphBack.DrawImageUnscaled(tileImages[playfieldData[x,y]], x * 16, y*16);

			}
		}
			
		public void Reset()
		{
			playfieldData = new int[width, height];
			numPellets = initNumPellets;

			for(int y=0; y<height; y++)
			{
				for(int x=0; x<width; x++)
				{
					playfieldData[x,y] = playfieldInitData[x,y];
				}
			}
		}

		public Direction GetGhostReturnDir(int x, int y)
		{
			return ghostReturnData[x,y];
		}

		public void ClearTile(int x, int y)
		{
			playfieldData[x,y] = (int)Tile.empty;
		}

		// NOTE: This function expects (int)Tile.wallSingle < (int)Tile.wall* < (int)Tile.wallEWS
		public bool IsWall(int x, int y)
		{
			return (playfieldData[x,y] >= (int)Tile.wallSingle && playfieldData[x,y] <= (int)Tile.wallEWS);
		}

		public bool IsGhostDoor(int x, int y)
		{
			return playfieldData[x,y] == (int)Tile.ghostDoor;
		}

		public bool IsPellet(int x, int y)
		{
			return playfieldData[x,y] == (int)Tile.pellet;
		}

		public bool IsPowerPellet(int x, int y)
		{
			return playfieldData[x,y] == (int)Tile.powerPellet;
		}
	}
}
