using System;
using System.IO;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Text;

namespace GDIPac
{
	/// <summary>
	/// Summary description for Sprite.
	/// </summary>
	public abstract class Sprite
	{
		private int animDelay; // Always either 2 or 3
		private int animState=0;
		private ImageSet imageSet;
		private IntPtr picBoxHandle;
		private int x, y; // In tiles, not pixels
		private Direction moveDirection;
		private Direction faceDirection;
		private Direction prevDirection;
		private bool canPassThroughDoor;

		protected Playfield playfield;

		#region Accessors
		public int X
		{
			get{ return x; }
			set{ x = value; }
		}
		public int Y
		{
			get{ return y; }
			set{ y = value; }
		}
		public int PixelsX
		{
			get{ return x*16 + (Util.GetAdjacentX(0, moveDirection) * AnimOffset); }
		}
		public int PixelsY
		{
			get{ return y*16 + (Util.GetAdjacentY(0, moveDirection) * AnimOffset); }
		}
		private int AnimOffset
		{
			get
			{
				 // Basically a simple "Lerp" between 0 and 15 
				return (16/animDelay)*animState;
			}
		}
		public Direction CurrDirection
		{
			get{ return moveDirection; }
		}
		#endregion
	
		public Sprite(IntPtr picBoxHandle, Playfield playfield, ImageSet imageSet, Direction initialDirection, bool canPassThroughDoor, bool isFast)
		{
			this.picBoxHandle = picBoxHandle;
			this.playfield = playfield;
			this.imageSet = imageSet;
			this.moveDirection = initialDirection;
			this.faceDirection = initialDirection;
			this.prevDirection = initialDirection;
			this.canPassThroughDoor = canPassThroughDoor;

			if(isFast)
				SetAnimFast();
			else
				SetAnimSlow();
		}

		private void StopIfAtWall()
		{
			Util.TileActingDelgateB isWall = new Util.TileActingDelgateB(playfield.IsWall);
			Util.TileActingDelgateB isGhostDoor = new Util.TileActingDelgateB(playfield.IsGhostDoor);

			if(Util.ApplyAdjacentB(isWall, x, y, moveDirection) ||
				(!canPassThroughDoor && Util.ApplyAdjacentB(isGhostDoor, x, y, moveDirection)) )
			{
				moveDirection = Direction.None;
			}
		}

		public void SetAnimFast()
		{
			animDelay = 2;
		}

		public void SetAnimSlow()
		{
			animDelay = 3;
		}

		protected abstract Direction ChooseNextDirection();
		protected abstract void DoExtraProcessing(bool isAnimAtZero);

		public void Draw()
		{
			using(Graphics graphBack = Graphics.FromHwnd(picBoxHandle))
			{
				graphBack.DrawImageUnscaled(imageSet.GetImage(faceDirection, animState), PixelsX, PixelsY);
			}
		}

		public void Erase()
		{
			Util.TileActingDelgateV drawAt = new Util.TileActingDelgateV(playfield.DrawAt);

			playfield.DrawAt(x, y);
			if(animState != 0)
			{
				Util.ApplyAdjacentV(drawAt, x, y, moveDirection);
			}
			else
			{
				Util.ApplyAdjacentV(drawAt, x, y, Util.GetReverseDirection(prevDirection));
			}
		}

		public void SetImageSet(ImageSet imageSet)
		{
			this.imageSet = imageSet;
		}

		public void DoFrame()
		{
			animState++;
			if(animState >= animDelay)
			{
				animState=0;

				x = Util.GetAdjacentX(x, moveDirection);
				y = Util.GetAdjacentY(y, moveDirection);

				prevDirection = moveDirection;
				moveDirection = ChooseNextDirection();

				if(moveDirection != Direction.None)
					faceDirection = moveDirection;

				StopIfAtWall();
			}

			DoExtraProcessing(animState==0);
		}
	}
}
